﻿namespace WebDevUtilities.VsixPkg
{
    using Microsoft.VisualStudio.Shell;
    using Microsoft.VisualStudio.Shell.Interop;
    using System;
    using System.ComponentModel.Design;
    using System.Diagnostics;
    using System.Globalization;
    using System.Runtime.InteropServices;
    using Extensions.CssSpriteGenerator;
    using Extensions.AspTemplateConverter;

    [PackageRegistration(UseManagedResourcesOnly = true)]
    // This attribute is used to register the information needed to show this package
    // in the Help/About dialog of Visual Studio.
    [InstalledProductRegistration("#110", "#112", "1.0", IconResourceID = 400)]
    // This attribute is needed to let the shell know that this package exposes some menus.
    [ProvideMenuResource("Menus.ctmenu", 1)]
    // This attribute registers a tool window exposed by this package.
    [ProvideToolWindow(typeof(CssSpriteGeneratorToolWindowPane))]
    [ProvideToolWindow(typeof(AspTemplateConverterToolWindowPane))]
    [Guid(GuidList.guidWebDevUtilitiesPkg)]
    public sealed class WebDevloperUtilitiesPackage : Package
    {
        public WebDevloperUtilitiesPackage()
        {
            Debug.WriteLine(string.Format(CultureInfo.CurrentCulture, "Entering constructor for: {0}", this.ToString()));
        }

        #region Package Members
        protected override void Initialize()
        {
            Debug.WriteLine(string.Format(CultureInfo.CurrentCulture, "Entering Initialize() of: {0}", this.ToString()));
            base.Initialize();
            OleMenuCommandService mcs = GetService(typeof(IMenuCommandService)) as OleMenuCommandService;
            if (null != mcs)
            {
                CommandID menuCmdID = new CommandID(GuidList.guidWebDevUtilitiesCmdSet, (int)PkgCmdIDList.cmdidCssSpriteGeneratorToolsMenu);
                MenuCommand menuCmd = new MenuCommand(ShowCssSpriteGeneratorToolWindow, menuCmdID);
                mcs.AddCommand(menuCmd);
                menuCmdID = new CommandID(GuidList.guidWebDevUtilitiesCmdSet, (int)PkgCmdIDList.cmdidCssSpriteGeneratorSolutionExplorerMenu);
                menuCmd = new MenuCommand(ShowCssSpriteGeneratorToolWindow, menuCmdID);
                mcs.AddCommand(menuCmd);
                menuCmdID = new CommandID(GuidList.guidWebDevUtilitiesCmdSet, (int)PkgCmdIDList.cmdidCssSpriteGeneratorQuickSolutionExplorerMenu);
                menuCmd = new MenuCommand(CssSpriteGeneratorQuickCmd, menuCmdID);
                mcs.AddCommand(menuCmd);
                menuCmdID = new CommandID(GuidList.guidWebDevUtilitiesCmdSet, (int)PkgCmdIDList.cmdidCssSpriteGeneratorViewMenu);
                menuCmd = new MenuCommand(ShowCssSpriteGeneratorToolWindow, menuCmdID);
                mcs.AddCommand(menuCmd);
                menuCmdID = new CommandID(GuidList.guidWebDevUtilitiesCmdSet, (int)PkgCmdIDList.cmdidAspTemplateConverterToolsMenu);
                menuCmd = new MenuCommand(ShowAspTemplateConverterToolWindow, menuCmdID);
                mcs.AddCommand(menuCmd);
                menuCmdID = new CommandID(GuidList.guidWebDevUtilitiesCmdSet, (int)PkgCmdIDList.cmdidAspTemplateConverterSolutionExplorerMenu);
                menuCmd = new MenuCommand(ShowAspTemplateConverterToolWindow, menuCmdID);
                mcs.AddCommand(menuCmd);
                menuCmdID = new CommandID(GuidList.guidWebDevUtilitiesCmdSet, (int)PkgCmdIDList.cmdidAspTemplateConverterQuickSolutionExplorerMenu);
                menuCmd = new MenuCommand(AspTemplateConverterQuickCmd, menuCmdID);
                mcs.AddCommand(menuCmd);
                menuCmdID = new CommandID(GuidList.guidWebDevUtilitiesCmdSet, (int)PkgCmdIDList.cmdidAspTemplateConverterViewMenu);
                menuCmd = new MenuCommand(ShowAspTemplateConverterToolWindow, menuCmdID);
                mcs.AddCommand(menuCmd);
                menuCmdID = new CommandID(GuidList.guidWebDevUtilitiesCmdSet, (int)PkgCmdIDList.cmdidCodeWasherSolutionExplorerMenu);
                menuCmd = new MenuCommand(RunCodeWasher, menuCmdID);
                mcs.AddCommand(menuCmd);
            }
        }
        #endregion

        #region Package Commands
        private void ShowCssSpriteGeneratorToolWindow(object sender, EventArgs e)
        {
            ToolWindowPane window = this.FindToolWindow(typeof(CssSpriteGeneratorToolWindowPane), 0, true);
            if ((null == window) || (null == window.Frame))
            {
                throw new NotSupportedException(Properties.Resources.CanNotCreateWindow);
            }
            IVsWindowFrame windowFrame = (IVsWindowFrame)window.Frame;
            Microsoft.VisualStudio.ErrorHandler.ThrowOnFailure(windowFrame.Show());
        }

        private void CssSpriteGeneratorQuickCmd(object sender, EventArgs e)
        {
            ToolWindowPane window = this.FindToolWindow(typeof(CssSpriteGeneratorToolWindowPane), 0, true);
            if ((null == window) || (null == window.Frame))
            {
                throw new NotSupportedException(Properties.Resources.CanNotCreateWindow);
            }
            IVsWindowFrame windowFrame = (IVsWindowFrame)window.Frame;
            Microsoft.VisualStudio.ErrorHandler.ThrowOnFailure(windowFrame.Show());
        }

        private void ShowAspTemplateConverterToolWindow(object sender, EventArgs e)
        {
            ToolWindowPane window = this.FindToolWindow(typeof(AspTemplateConverterToolWindowPane), 0, true);
            if ((null == window) || (null == window.Frame))
            {
                throw new NotSupportedException(Properties.Resources.CanNotCreateWindow);
            }
            IVsWindowFrame windowFrame = (IVsWindowFrame)window.Frame;
            Microsoft.VisualStudio.ErrorHandler.ThrowOnFailure(windowFrame.Show());
        }

        private void AspTemplateConverterQuickCmd(object sender, EventArgs e)
        {
            ToolWindowPane window = this.FindToolWindow(typeof(AspTemplateConverterToolWindowPane), 0, true);
            if ((null == window) || (null == window.Frame))
            {
                throw new NotSupportedException(Properties.Resources.CanNotCreateWindow);
            }
            IVsWindowFrame windowFrame = (IVsWindowFrame)window.Frame;
            Microsoft.VisualStudio.ErrorHandler.ThrowOnFailure(windowFrame.Show());
        }

        private void RunCodeWasher(object sender, EventArgs e)
        {
            TestCall(sender, e);
        }

        private void TestCall(object sender, EventArgs e)
        {
            IVsUIShell uiShell = (IVsUIShell)GetService(typeof(SVsUIShell));
            Guid clsid = Guid.Empty;
            int result;
            Microsoft.VisualStudio.ErrorHandler.ThrowOnFailure(uiShell.ShowMessageBox(
                       0,
                       ref clsid,
                       "Test",
                       string.Format(CultureInfo.CurrentCulture, "Inside {0}.MenuItemCallback()", this.ToString()),
                       string.Empty,
                       0,
                       OLEMSGBUTTON.OLEMSGBUTTON_OK,
                       OLEMSGDEFBUTTON.OLEMSGDEFBUTTON_FIRST,
                       OLEMSGICON.OLEMSGICON_INFO,
                       0,        // false
                       out result));
        }
        #endregion
    }
}
